require 'spec_helper'

describe Rustle::Color do

  let(:color) { Rustle::Color.rgb(128,128,128) }

  context "when initialized" do
    let(:color_with_invalid_params) { Rustle::Color.new(-100,1000,0) }

    it "corrects negative rgb values to zero" do
      expect(color_with_invalid_params.r).to eq(0)
    end

    it "corrects very large rgb values to 255" do
      expect(color_with_invalid_params.g).to eq(255)
    end

    it "makes no changes to rgb values between 0 and 255" do
      color1 = Rustle::Color.new(0,0,0)
      expect(color1.r).to eq(0)
      expect(color1.g).to eq(0)
      expect(color1.b).to eq(0)

      color2 = Rustle::Color.new(255,255,255)
      expect(color2.r).to eq(255)
      expect(color2.g).to eq(255)
      expect(color2.b).to eq(255)
    end
  end

  describe "#to_a" do
    it "returns an array" do
      expect(color.to_a).to be_a(Array)
    end

    it "the array has the correct [r,g,b] values" do
      expect(color.to_a).to eq([128,128,128])
    end

    it "rounds decimal values down to the nearest whole number" do
      fractional_rgbs = Rustle::Color.rgb(0.5,10.7,99.9)
      expect(fractional_rgbs.to_a).to eq([0, 10, 99])
    end
  end

  describe "#eql?" do
    let(:colorA) { Rustle::Color.rgb(128,128,128) }
    let(:another_colorA) { Rustle::Color.rgb(128,128,128) }
    let(:colorB) { Rustle::Color.rgb(0, 0, 0) }

    it "returns true if the reciever and parameter objects hold equivalent rgb values" do
      expect(colorA.eql?(another_colorA)).to be_true
    end

    it "returns false if the reciever and parameter objects differ in rgb values" do
      expect(colorA.eql?(colorB)).to be_false
    end
  end

  describe "Rustle::Color.hex factory method" do
    let(:badass_rgb) { Rustle::Color.rgb(186, 218, 85) }

    context "understands hex color codes WITHOUT a leading hash" do
      let (:badass) { Rustle::Color.hex "BADA55" }

      it "creates a color object" do
        expect(badass).to be_a(Rustle::Color)
      end

      it "has the proper rgb values" do
        expect(badass).to eql(badass_rgb)
      end
    end

    context "understands hex color codes WITH a leading hash" do
      let(:badass) { Rustle::Color.hex "#BADA55" }

      it "creates a color object" do
        expect(badass).to be_a(Rustle::Color)
      end

      it "has the right rgb values" do
        expect(badass).to eql(badass_rgb)
      end
    end

    context "understands hex color codes in three character format" do
      let(:bad_hex) { Rustle::Color.hex "#BAD" }
      let(:bad_rgb) { Rustle::Color.rgb 187, 170, 221 }

      it "creates a color object" do
        expect(bad_hex).to be_a(Rustle::Color)
      end

      it "should have the correct rgb values" do
        expect(bad_hex).to eql(bad_rgb)
      end
    end

    it "is case insensitive" do
      first_badass = Rustle::Color.hex "#BADa55"
      second_badass = Rustle::Color.hex "#badA55"
      expect(first_badass).to eql(second_badass)
    end

    it "raises an error when given an invalid color code string" do
      expect { Rustle::Color.hex "#ABCDEFABCDEF" }.to raise_error(Rustle::InvalidHexColorCode)
    end
  end

  describe "Rustle::Color.hsb factory method" do
    let(:rgb) { Rustle::Color.rgb 187, 170, 221 }
    let(:hsb) { Rustle::Color.hsb 260, 0.23, 0.866 }
    # Color: #BBAADD
    # RGB: (187, 170, 221)
    # HSV: 260 (260), saturation: 23 (23.0769), value: 87 (86.6667)

    it "produces a color object" do
      expect(hsb).to be_a(Rustle::Color)
    end

    it "has the correct rgb properties" do
      # Use be_within to give some allowance for rounting errors
      expect(hsb.r).to be_within(3).of(rgb.r)
      expect(hsb.g).to be_within(3).of(rgb.g)
      expect(hsb.b).to be_within(3).of(rgb.b)
    end
  end

  describe "#to_hsb" do
    let(:hsb) { Rustle::Color.hsb 180, 0.5, 0.8 }

    it "returns an array of [h,s,v] coordinates" do
      expect(color.to_hsb).to be_a(Array)
    end

    it "should have the (approximate) [h,s,v] coordinates" do
      hsb_arr = hsb.to_hsb
      expect(hsb_arr[0]).to be_within(1).of(180)
      expect(hsb_arr[1]).to be_within(0.1).of(0.5)
      expect(hsb_arr[2]).to be_within(0.1).of(0.8)
    end

  end
end
